<?php

// Exit if accessed directly
if ( ! defined( 'ABSPATH' ) ) exit;

/**
 * The main class
 *
 * @since 1.0.0
 */
class WpGetApi_Extras_Extend {

    //public $encryption = '';

    /**
     * Main constructor
     *
     * @since 1.0.0
     *
     */
    public function __construct() {

        // caching
        add_filter( 'wpgetapi_fields_endpoints', array( $this, 'add_cache_field' ), 10, 1 );
        add_filter( 'wpgetapi_before_get_request', array( $this, 'maybe_cache' ), 10, 2 );

        // add new results format
        add_filter( 'wpgetapi_results_format_options', array( $this, 'results_format_options' ), 10, 2 );

        // get nested data
        add_filter( 'wpgetapi_json_response_body', array( $this, 'nested_data' ), 10, 2 );

        // maybe output as xml
        add_filter( 'wpgetapi_raw_data', array( $this, 'output_xml' ), 10, 2 );

        // maybe do variables on endpoint
        add_filter( 'wpgetapi_final_url', array( $this, 'variables_on_endpoint' ), 10, 2 );

        // maybe add custom query variables - useful within shortcode
        add_filter( 'wpgetapi_final_url', array( $this, 'custom_query_variables' ), 11, 2 );

    }


    /**
     * Maybe do variables on endpoint
     */
    public function variables_on_endpoint( $url, $api ) {

        // if we have endpoint variables set, proceed
        if( isset( $api->args['endpoint_variables'] ) && ! empty( $api->args['endpoint_variables'] ) ) {

            foreach ( $api->args['endpoint_variables'] as $index => $var ) {

                if ( strpos( $api->endpoint, '{' . $index . '}' ) !== false ) {

                    $url = str_replace( '{' . $index . '}', $var, $url );

                }

            }

        }

        return $url;
    }


    /**
     * Maybe do variables on endpoint
     */
    public function custom_query_variables( $url, $api ) {

        // if we have endpoint variables set, proceed
        if( isset( $api->args['query_variables'] ) && ! empty( $api->args['query_variables'] ) ) {

            $vars = explode(',', $api->args['query_variables'] );
            $finalArray = array();

            if( $vars && is_array( $vars ) ) {
                
                foreach ($vars as $var) {
                    $couple = explode('=', $var);
                    $finalArray[$couple[0]] = $couple[1];
                }

            } else {

                $couple = explode( '=', $api->args['query_variables'] );
                $finalArray[$couple[0]] = $couple[1];

            }


            $url = add_query_arg( $finalArray, $url );

        }

        return $url;
    }


    /**
     * Add new options to result format
     */
    public function results_format_options( $options ) {
        $new = array(
            'xml_string' => __( 'XML (as string)', 'wpgetapi' ),
            'xml_array' => __( 'XML (as array data)', 'wpgetapi' ),
        );
        $options = array_merge( $options, $new );
        return $options;
    }


    /**
     * Output in XML
     */
    public function output_xml( $data, $api ) {

        // returning in XML string format
        if( $api->results_format == 'xml_string' ) {
            return wp_kses_post( $data );
        }

        // returning in XML string format
        if( $api->results_format == 'xml_array' ) {
            $data = simplexml_load_string( $data );
            // converts all data to arrays, removing objects
            $data = json_decode( json_encode( $data ), true );
            $data = apply_filters( 'wpgetapi_json_response_body', $data, $api->keys );
            return $data;
        }

        return $data;

    }


    /**
     * Setup the cache field
     */
    public function add_cache_field( $field ) {

        $insert = array(
            'name' => __( 'Cache Time', 'wpgetapi' ),
            'id'   => 'cache_time',
            'type' => 'text',
            'classes' => 'field-cache-time',
            'attributes' => array( 
                'placeholder' => '',
            ),
            'desc' => 'How long (in seconds) to cache requests. Only for GET method. ',
        );

        $field = $this->array_insert( $field, 4, $insert );
            
        return $field;

    }


    public function array_insert( $array, $position, $insert ) {
        if ($position > 0) {
            if ($position == 1) {
                array_unshift($array, array());
            } else {
                $position = $position - 1;
                array_splice($array, $position, 0, array(
                    ''
                ));
            }
            $array[$position] = $insert;
        }

        return $array;
    }


    /**
     * Setup and do caching if set
     */
    public function maybe_cache( $response, $api ) {

        $api->cache_time = 30;
        if( isset( $api->cache_time ) && $api->cache_time > 0 ) {

            $transient_name = 'wpgetapi_' . $api->api_id . '_' . $api->endpoint_id;

            // Do we have this information in our transients already?
            $transient = get_transient( $transient_name );
              
            // Yep!  Just return it and we're done.
            if( ! empty( $transient ) ) {
                
                // The function will return here every time after the first time it is run, until the transient expires.
                return $transient;

            } 

        }

        $response = wp_remote_get( $api->final_url, $api->final_headers );

        if( isset( $api->cache_time ) && $api->cache_time > 0 ) {
                
            // Don't bother caching stuff we don't need
            if( 
                is_array( $response ) && 
                ! is_wp_error( $response ) && 
                isset( $response['response'] ) && 
                isset( $response['response']['code'] )
            ) { 
                // Save the API response
                set_transient( $transient_name, $response, apply_filters( 'wpgetapi_cache_time', $api->cache_time ) );
            }
            
        }

        return $response;

    }


    /**
     * Get the nested data
     */
    public function nested_data( $data = array(), $keys = array() ) {

        // if we have keys
        if( $keys && is_array( $keys ) ) {
            
            $count = count( $keys );
            $keys = wpgetapi_sanitize_text_or_array( $keys );

            // using a switch statement until find better solution
            switch ( $count ) {
                case 1:
                    $data = $data[ $keys[0] ];
                    break;
                case 2:
                    $data = $data[ $keys[0] ][ $keys[1] ];
                    break;
                case 3:
                    $data = $data[ $keys[0] ][ $keys[1] ][ $keys[2] ];
                    break;
                case 4:
                    $data = $data[ $keys[0] ][ $keys[1] ][ $keys[2] ][ $keys[3] ];
                    break;
                case 5:
                    $data = $data[ $keys[0] ][ $keys[1] ][ $keys[2] ][ $keys[3] ][ $keys[4] ];
                    break;
                case 6:
                    $data = $data[ $keys[0] ][ $keys[1] ][ $keys[2] ][ $keys[3] ][ $keys[4] ][ $keys[5] ];
                    break;
                case 7:
                    $data = $data[ $keys[0] ][ $keys[1] ][ $keys[2] ][ $keys[3] ][ $keys[4] ][ $keys[5] ][ $keys[6] ];
                    break;
                case 8:
                    $data = $data[ $keys[0] ][ $keys[1] ][ $keys[2] ][ $keys[3] ][ $keys[4] ][ $keys[5] ][ $keys[6] ][ $keys[7] ];
                    break;
                case 9:
                    $data = $data[ $keys[0] ][ $keys[1] ][ $keys[2] ][ $keys[3] ][ $keys[4] ][ $keys[5] ][ $keys[6] ][ $keys[7] ][ $keys[8] ];
                    break;
                case 10:
                    $data = $data[ $keys[0] ][ $keys[1] ][ $keys[2] ][ $keys[3] ][ $keys[4] ][ $keys[5] ][ $keys[6] ][ $keys[7] ][ $keys[8] ][ $keys[9] ];
                    break;
            }

        }

        return $data;

    }


}

return new WpGetApi_Extras_Extend();